/*
  aruli.h
 */

#ifndef ARULI_H
#define ARULI_H

#include <pthread.h>

#include "ruli.h"

typedef enum {
  ARULI_QS_VOID     = 0,
  ARULI_QS_DONE     = 1 << 0,
  ARULI_QS_WAITABLE = 1 << 1
} aruli_qs_t;

typedef void (*aruli_callback_t)(void *);

typedef struct {
  struct ruli_t    *ruli_srv_rec;

  aruli_callback_t query_callback;
  void             *query_callback_param;
  int              query_result;
  aruli_qs_t       query_status;
  pthread_t        query_thread_id;

  const char       *query_param_service;
  const char       *query_param_proto;
  const char       *query_param_dname;
  int              query_param_default_port;
} aruli_t;

int aruli_srv_query_done(aruli_t *aruli_srv_rec);
int aruli_srv_query_result(aruli_t *aruli_srv_rec);
int aruli_srv_query_wait(aruli_t *aruli_srv_rec);

int aruli_srv_make(aruli_t **aruli_srv_rec);
void aruli_srv_reset(aruli_t *aruli_srv_rec);
int aruli_srv_destroy(aruli_t *aruli_srv_rec);
int aruli_srv_query(aruli_t *aruli_srv_rec, int waitable, aruli_callback_t query_callback, void *callback_param, const char *service, const char *proto, const char *dname, int default_port);

void aruli_srv_target_reset(aruli_t *aruli_srv_rec);
int aruli_srv_target_pending(aruli_t *aruli_srv_rec);
int aruli_srv_target_next(aruli_t *aruli_srv_rec, struct ruli_srv_target_t **srv_target);

int aruli_http_srv_query(aruli_t *aruli_srv_rec, int waitable, aruli_callback_t query_callback, void *callback_param, const char *dname, int default_port);

#endif /* ARULI_H */

/*
 * eof: ruli.h
 */

