/*
  ruli.h
 */

#ifndef RULI_H
#define RULI_H

#include "ruli/ruli_list.h"

#define RULI_DNAME_BUFSZ 256
#define RULI_ADDR_BUFSZ  256
#define RULI_MSG_BUFSZ   1024

/*
 * SRV status
 */
enum {
  RULI_SRV_ST_VOID,     /* invalid status */
  RULI_SRV_ST_NEXT,     /* user can request next tuple (address, port) */
  RULI_SRV_ST_ABORT     /* user can't request tuples */
};

/*
 * QR: query result type
 */
enum {
  RULI_QR_OK = 0,            /* ok */
  RULI_QR_UNKNOWN,           /* unknown error: FIX: must be specified! */
  RULI_QR_QT_NOTFOUND,       /* query type not found */
  RULI_QR_QC_NOTFOUND,       /* query class not found */
  RULI_QR_RINIT_FAILED,      /* resolver init failed */
  RULI_QR_RSEARCH_FAILED,    /* resolver search failed */
  RULI_QR_SRV_BADNAME,       /* bad srv name */
  RULI_QR_SRV_BADDATAPTR,    /* bad srv data pointer */
  RULI_QR_LIST_INIT,         /* list init failed */
  RULI_QR_SRV_PARSE_MALLOC,  /* srv parse malloc failed */
  RULI_QR_NAME_OVERFLOW,     /* parse name overflow */
  RULI_QR_LIST_ERROR,        /* list error */
  RULI_QR_SRV_TARGET_EMPTY,  /* srv target list empty */
  RULI_QR_SRV_USER_ABORT,    /* service not available */
  RULI_QR_SRV_BAD_STATUS,    /* bad srv status */
  RULI_QR_A_PARSE_MALLOC,    /* addr parse malloc failed */
  RULI_QR_TADDR_MALLOC,      /* target address malloc failed */
  RULI_QR_BAD_RCODE,         /* bad rcode */
  RULI_QR_MALLOC,            /* generic malloc failed */

  RULI_QR_ARULI_THR_INIT,    /* asynchronous thread init failed */
  RULI_QR_ARULI_THR_DETACH,  /* asynchronous thread detach failed */
  RULI_QR_ARULI_THR_CREATE,  /* asynchronous thread create failed */
  RULI_QR_ARULI_THR_DESTROY, /* asynchronous thread destroy failed */
  RULI_QR_ARULI_THR_JOIN     /* asynchronous thread join failed */
};

/*
 * From RFC 1035
 */
enum {
  RULI_RCODE_NOERROR        = 0, /* No error condition */
  RULI_RCODE_FORMATERROR    = 1, /* Format error */
  RULI_RCODE_SERVERFAILURE  = 2, /* Server failure */
  RULI_RCODE_NAMEERROR      = 3, /* Name error */
  RULI_RCODE_NOTIMPLEMENTED = 4, /* Not implemented */
  RULI_RCODE_REFUSED        = 5  /* Refused */
};

/*
 * From RFC 1035
 */
enum {
  RULI_RR_TYPE_A     = 1,   /* 1 a host address */
  RULI_RR_TYPE_NS    = 2,   /* 2 an authoritative name server */
  RULI_RR_TYPE_MD    = 3,   /* 3 a mail destination (Obsolete - use MX) */
  RULI_RR_TYPE_MF    = 4,   /* 4 a mail forwarder (Obsolete - use MX) */
  RULI_RR_TYPE_CNAME = 5,   /* 5 the canonical name for an alias */
  RULI_RR_TYPE_SOA   = 6,   /* 6 marks the start of a zone of authority */
  RULI_RR_TYPE_MB    = 7,   /* 7 a mailbox domain name (EXPERIMENTAL) */
  RULI_RR_TYPE_MG    = 8,   /* 8 a mail group member (EXPERIMENTAL) */
  RULI_RR_TYPE_MR    = 9,   /* 9 a mail rename domain name (EXPERIMENTAL) */
  RULI_RR_TYPE_NULL  = 10,  /* 10 a null RR (EXPERIMENTAL) */
  RULI_RR_TYPE_WKS   = 11,  /* 11 a well known service description */
  RULI_RR_TYPE_PTR   = 12,  /* 12 a domain name pointer */
  RULI_RR_TYPE_HINFO = 13,  /* 13 host information */
  RULI_RR_TYPE_MINFO = 14,  /* 14 mailbox or mail list information */
  RULI_RR_TYPE_MX    = 15,  /* 15 mail exchange */
  RULI_RR_TYPE_TXT   = 16,  /* 16 text strings */
  RULI_RR_TYPE_SRV   = 33,  /* 33 location of service */
  RULI_RR_TYPE_AXFR  = 252, /* 252 a request for a transfer of an entire zone */
  RULI_RR_TYPE_MAILB = 253, /* 253 a request for mailbox-related records (MB, MG or MR) */
  RULI_RR_TYPE_MAILA = 254, /* 254 a request for mail agent RRs (Obsolete - see MX) */
  RULI_RR_TYPE_ANY   = 255  /* 255 a request for all records */
};

/*
 * From RFC 1035
 */
enum {
  RULI_RR_CLASS_IN  = 1,  /* 1 the Internet */
  RULI_RR_CLASS_CS  = 2,  /* 2 the CSNET class (Obsolete) */
  RULI_RR_CLASS_CH  = 3,  /* 3 the CHAOS class */
  RULI_RR_CLASS_HS  = 4,  /* 4 Hesiod (Dyer 87) */
  RULI_RR_CLASS_ANY = 255 /* 255 any class */
};

struct ruli_qtype_pair {
  char *qt_name;
  int qt_code;
};

struct ruli_qclass_pair {
  char *qc_name;
  int qc_code;
};

/*
 * Misc. utilities
 */
int ruli_get_qtype_code(const char *qt_name);
int ruli_get_qclass_code(const char *qc_name);
const char *ruli_get_qtype_name(int qt_code);
const char *ruli_get_qclass_name(int qc_code);

/*
 * Query facilities
 */
const char *ruli_qr_str(int code);
int ruli_qry(const char *dname, int qt_code, int qc_code, char *answer, int ans_size, int *ans_len);
int ruli_query(const char *dname, const char *qt_name, const char *qc_name, char *answer, int ans_size, int *ans_len);

typedef unsigned char  ruli_uint8_t;
typedef unsigned short ruli_uint16_t;
typedef unsigned int   ruli_uint32_t;

struct ruli_msg_hdr_t {
  ruli_uint8_t  rcode;
  ruli_uint16_t qdcount;
  ruli_uint16_t ancount;
  ruli_uint16_t nscount;
  ruli_uint16_t arcount;
};

struct ruli_rr_t {
  ruli_uint8_t  name[RULI_DNAME_BUFSZ];
  ruli_uint8_t  name_len;
  ruli_uint16_t type;
  ruli_uint16_t class;
  ruli_uint32_t ttl;
  ruli_uint16_t rdlength;
  ruli_uint8_t  *rdata;
};

struct ruli_rr_addr_t {
  ruli_uint8_t name[RULI_DNAME_BUFSZ];
  ruli_uint8_t addr[RULI_ADDR_BUFSZ];
  ruli_uint8_t addr_len;
};

struct ruli_rr_srv_t {
  ruli_uint8_t  name[RULI_DNAME_BUFSZ];
  ruli_uint16_t priority;
  ruli_uint16_t weight;
  ruli_uint16_t port;
  ruli_uint8_t  target[RULI_DNAME_BUFSZ];
};

struct ruli_t {
  /*
   * message header
   */
  struct ruli_msg_hdr_t msg_hdr;

  /* 
   * internal status 
   */
  int                   srv_status;

  /* 
   * addit_addr_list (ruli_rr_addr_t): additional A RR list 
   */
  struct ruli_list_t    addit_addr_list; 

  /* 
   *  res_addr_list (ruli_rr_addr_t): resolver A RR list 
   *  (for A RRs not found in additional section) 
   */
  struct ruli_list_t    res_addr_list; 

  /* 
   *  srv_list (ruli_rr_srv_t): store SRV RRs 
   */
  struct ruli_list_t    srv_list; 

  /* 
   * target_list (ruli_rr_srv_t): SRV RR list sorted by priority/weight 
   */
  struct ruli_list_t    target_list;

  /* 
   * addr_list (ruli_srv_target_t): target addr list issued to user 
   */
  struct ruli_list_t    target_addr_list;
  size_t                target_addr_list_curr_index;
};

/*
 * High-level facilities
 */

struct ruli_srv_target_t {
  const char          *name;    /* target name (null-terminated string) */
  const unsigned char *addr;    /* target address */
  int                 addr_len; /* address length */
  int                 port;     /* target port */
};

int ruli_srv_make(struct ruli_t **ruli_srv_rec);
void ruli_srv_reset(struct ruli_t *ruli_srv_rec);
int ruli_srv_destroy(struct ruli_t *ruli_srv_rec);
int ruli_srv_query(struct ruli_t *ruli_srv_rec, const char *service, const char *proto, const char *dname, int default_port);

void ruli_srv_target_reset(struct ruli_t *ruli_srv_rec);
int ruli_srv_target_pending(struct ruli_t *ruli_srv_rec);
int ruli_srv_target_next(struct ruli_t *ruli_srv_rec, struct ruli_srv_target_t **srv_target);

int ruli_http_srv_query(struct ruli_t *ruli_srv_rec, const char *dname, int default_port);

#endif /* RULI_H */

/*
 * eof: ruli.h
 */

